! ***********************************************************************
!
!   Copyright (C) 2011  Bill Paxton
!
!   this file is part of mesa.
!
!   mesa is free software; you can redistribute it and/or modify
!   it under the terms of the gnu general library public license as published
!   by the free software foundation; either version 2 of the license, or
!   (at your option) any later version.
!
!   mesa is distributed in the hope that it will be useful, 
!   but without any warranty; without even the implied warranty of
!   merchantability or fitness for a particular purpose.  see the
!   gnu library general public license for more details.
!
!   you should have received a copy of the gnu library general public license
!   along with this software; if not, write to the free software
!   foundation, inc., 59 temple place, suite 330, boston, ma 02111-1307 usa
!
! ***********************************************************************
      module run_star_extras

      use star_lib
      use star_def
      use const_def
      
      implicit none
      
      integer :: time0, time1, clock_rate
!cxh
      integer :: model_fre
!cxh      
      real(dp), parameter :: expected_runtime = 1 ! minutes

      
      ! these routines are called by the standard run_star check_model
      contains
      
      
      subroutine extras_controls(s, ierr)
      	use adipls_support, only: &
      	   astero_other_procs, use_other_adipls_mode_info
         type (star_info), pointer :: s
         integer, intent(out) :: ierr
         ierr = 0
         write(*,*) 'extras_controls: set use_other_adipls_mode_info'
      	use_other_adipls_mode_info = .true.
      	astero_other_procs% other_adipls_mode_info => my_other_adipls_mode_info
      end subroutine extras_controls


      subroutine my_other_adipls_mode_info( &
            l, order, freq, inertia, x, y, aa, data, nn, iy, iaa, ispcpr, ierr)
!---
         type (star_info), pointer :: s
!---
         integer, intent(in) :: l, order
         real(dp), intent(in) :: freq, inertia
         real(dp), intent(in) :: x(1:nn), y(1:iy,1:nn), aa(1:iaa,1:nn), data(8)
         integer, intent(in) :: nn, iy, iaa, ispcpr
         integer, intent(out) :: ierr
!cxh---
         character(len=256)::stkame,pgame
         integer,parameter::fcxh1=2016,fcxh2=2017,fcxh3=2018
         integer :: i, j, k,korder,morder,kpg
         real(dp) :: belta,r,g,rh,Wgr,Wgh,tstar,xmr
!---
         kpg=order
         if(l.eq.0)kpg=kpg-1
!---output fre file
         write (pgame,"('fre',i6.6,'.txt')")model_fre
         call Ybelta(l, freq, x, y, aa, data(1), data(2), nn, iy, iaa,belta)
         open(fcxh2,file=pgame,position='append')
         write(fcxh2,"(3i8,2e20.10,f10.3)")model_fre,l,kpg,freq,inertia,belta
         close(fcxh2)
!---output eigenfunction
         if(model_fre.gt.1681.or.model_fre.lt.1681)goto 2000
!          goto 2000
!cxh---
         korder=kpg
         if(korder.ge.0)then
         write (stkame,2018)'St',model_fre,'L',l,'p',korder,'.dat'
         endif
         if(korder.lt.0)then
          morder=abs(korder)
         write (stkame,2018)'St',model_fre,'L',l,'g',morder,'.dat'
         endif
2018    format(A2,i4.4,A1,i2.2,A1,i3.3,A4)
         tstar=standard_cgrav*data(1)/data(2)**3d0 ! GM/R^3 
         open(fcxh1,file=stkame,position='append') 
         do i=2,nn
         xmr=aa(1,i)*x(i)**3d0
         r=x(i)*data(2)
         g=r*aa(1,i)*tstar
         rh=aa(5,i)*g/r/standard_cgrav/4d0/pi
         Wgr=y(1,i)*data(2)
         write(fcxh1,'(8(1pes27.16e3,1x))')r,x(i),xmr,rh,y(1,i),y(2,i),Wgr,((1d0-xmr)*xmr)**0.5d0*wgr
         enddo
         close(fcxh1)
!---
2000     continue
         ierr = 0
!cxh----
         !write(*,*) 'astero called my_other_adipls_mode_info'
      end subroutine my_other_adipls_mode_info
      
      subroutine Ybelta(l, freq, x, y, aa, starM, starR, nn, iy, iaa, belta)
        real,parameter:: sunm=1.9892d33,sunr=6.9598d10
        real,parameter:: gconst=6.67428d-8, pi=3.141592653589793d0
        real(dp), intent(in) :: freq,starR,starM
        real(dp), intent(in) :: x(1:nn), y(1:iy,1:nn), aa(1:iaa,1:nn)
        integer, intent(in) :: l, nn, iy, iaa

        real(dp) :: fl,fl2,tstar,Wgb1,Wgb11,Wgbe22,Wgbe2
        real(dp) :: r,g,rh,Wgr,Wgh,belta
        integer k
!c
        fl = dble(l)             
        fl2=fl*(fl+1d0)     
        tstar=gconst*starM/starR**3d0 ! GM/R^3    
!c
        Wgb1=0d0
        Wgb11=0d0
        Wgbe22=0d0
        Wgbe2=0d0
        do k=2,nn-1
        r=(x(k)+x(k+1))/2d0*starR            ! r in cm
        g=r*(aa(1,k)+aa(1,k+1))/2d0*tstar
        rh=(aa(5,k)+aa(5,k+1))/2d0*g/r/gconst/4d0/pi
        Wgr=(y(1,k)+y(1,k+1))/2d0*starR
        Wgh=(y(2,k)+y(2,k+1))/2d0*starR/(fl+1)/fl
!c
        Wgbe22=(x(k+1)-x(k))*r**2d0*rh*(Wgr**2d0+Wgh**2d0*(fl2-1d0)-2d0*Wgr*Wgh)
        Wgbe2=Wgbe2+Wgbe22
!c
        Wgb11=(x(k+1)-x(k))*r**2d0*rh*(Wgr**2d0+Wgh**2d0*fl2)
        Wgb1=Wgb11+Wgb1
        enddo ! k
        belta=Wgbe2/Wgb1
        if(l.eq.0)belta=0d0
        !write(*,*)'belta=',belta
      end subroutine Ybelta


      integer function extras_startup(s, id, restart, ierr)
         type (star_info), pointer :: s
         integer, intent(in) :: id
         logical, intent(in) :: restart
         integer, intent(out) :: ierr
         ierr = 0
         extras_startup = 0
         call system_clock(time0,clock_rate)
         if (.not. restart) then
            call alloc_extra_info(s)
         else ! it is a restart
            call unpack_extra_info(s)
         end if
      end function extras_startup
      
      
      subroutine extras_after_evolve(s, id, id_extra, ierr)
         type (star_info), pointer :: s
         integer, intent(in) :: id, id_extra
         integer, intent(out) :: ierr
         
         real(dp) :: dt, expected_freq, freq
         logical :: okay, store_for_adipls, save_mode_info
         integer :: l_to_match, order_to_match, order_to_save
         character (len=256) :: save_mode_filename
         
         ierr = 0
         
         store_for_adipls = .true.
         l_to_match = 0
         order_to_match = 999
         expected_freq = 6579d0
         save_mode_info = .true.
         order_to_save = 5
         save_mode_filename = 'eigen.data'
         
         call get_adipls_frequency_info( &
            s, store_for_adipls, l_to_match, order_to_match, expected_freq, &
            save_mode_info, order_to_save, save_mode_filename, freq, okay, ierr)
         if (ierr /= 0) stop 1
         if (okay) then
            write(*,'(a,2f20.2)') 'got ok match for expected frequency', freq, expected_freq
         else
            write(*,'(a,2f20.2)') 'ERROR: bad match for expected frequency', freq, expected_freq
         end if

         call system_clock(time1,clock_rate)
         dt = dble(time1 - time0) / clock_rate / 60
         if (dt > 10*expected_runtime) then
            write(*,'(/,a30,2f18.6,a,/)') '>>>>>>> EXCESSIVE runtime', &
               dt, expected_runtime, '   <<<<<<<<<  ERROR'
         else
            write(*,'(/,a50,2f18.6,99i10/)') 'runtime, retries, backups, steps', &
               dt, expected_runtime, s% num_retries, s% num_backups, s% model_number
         end if
         
      end subroutine extras_after_evolve
      

      ! returns either keep_going, retry, backup, or terminate.
      integer function extras_check_model(s, id, id_extra)
         type (star_info), pointer :: s
         integer, intent(in) :: id, id_extra

         real(dp) :: dt, expected_freq, freq,df,dp,duasy,atem,fsndlnr
         logical :: okay, store_for_adipls, save_mode_info
         integer :: ierr, l_to_match, order_to_match, order_to_save
         character (len=256) :: save_mode_filename
         integer :: k,m,model
         real :: vals(100000,9)
         character (len=256) :: filename= "Interval_FP.txt"
         real,parameter::fln10=2.302585092994046d0
         integer,parameter:: fileid=100,fcxh02=2017
!cxh
         real :: osc_g,osc_Teff,osc_R,osc_L,osc_XcH,Teff_uper,Teff_lower
         integer :: iosc0
!cxh
         extras_check_model = keep_going
!---
         model_fre=s% model_number
         osc_g=log10(standard_cgrav*Msun*s% star_mass/(Rsun*s% photosphere_r)**2d0)
         osc_Teff=s% Teff
         osc_XcH=s% center_h1
         Teff_lower=6800d0-200d0
         Teff_uper=7200d0+200d0
!---when to stop
         if(osc_XcH.lt.2d-8)then
          s% log_g_lower_limit = 3.495d0
         endif
!---timestep
         if(osc_XcH.lt.2d-8)s% varcontrol_target=5d-6
         if(osc_Teff.gt.Teff_uper.or.osc_Teff.lt.Teff_lower)s% varcontrol_target=2.5d-5
!---mesh adjustment
         if(osc_XcH.lt.2d-8)s% M_function_weight = 700        
!---error box
         if(osc_g.gt.4.0d0.or.osc_g.lt.3.5d0)goto 2011
         if(osc_Teff.gt.Teff_uper.or.osc_Teff.lt.Teff_lower)goto 2011
!---
         ierr = 0
!---calculate pulsation frequency for specific evolutionary model
         if(model_fre.lt.1681.or.model_fre.gt.1681)goto 2011
         !goto 2011
!---
1000     do l_to_match= 0, 2
         store_for_adipls = .true.
         order_to_match = 5
         expected_freq = 4433
         save_mode_info = .true.
         order_to_save = 10
         save_mode_filename = '4433.mod'
         
         call get_adipls_frequency_info( &
            s, store_for_adipls, l_to_match, order_to_match, expected_freq, &
            save_mode_info, order_to_save, save_mode_filename, freq, okay, ierr)
         if (ierr /= 0) extras_check_model = terminate
         end do

2011    continue   
      
      end function extras_check_model     
      
      subroutine get_adipls_frequency_info( &
            s, store_for_adipls, l_to_match, order_to_match, expected_freq, &
            save_mode_info, order_to_save, save_mode_filename, freq, okay, ierr)
         use adipls_support, only: adipls_get_one_el_info
         type (star_info), pointer :: s
         integer, intent(in) :: l_to_match, order_to_match, order_to_save
         logical, intent(in) :: store_for_adipls, save_mode_info
         character (len=*), intent(in) :: save_mode_filename
         real(dp), intent(in) :: expected_freq ! ignore if < 0
         real(dp), intent(out) :: freq
         logical, intent(out) :: okay ! true if expected_freq is okay
         integer, intent(out) :: ierr
         integer :: l, iscan, i, num
         real(dp) :: nu1, nu2, R, G, M
         real(dp), pointer, dimension(:) :: l_freq, l_inertia
         integer, pointer, dimension(:) :: l_order, l_em
         logical :: add_center_point, keep_surface_point, &
            add_atmosphere, do_restribute_mesh

         include 'formats'

         ierr = 0
         okay = .true.
         R = Rsun*s% photosphere_r
         G = s% cgrav(1)
         M = Msun*s% star_mass
         add_center_point = .true.
         keep_surface_point = .false.
         add_atmosphere = .true.
         do_restribute_mesh = .false.
         l = l_to_match
         
         nullify(l_freq)
         nullify(l_inertia)
         nullify(l_order)
         nullify(l_em)
         nu1 = 80
         nu2 = 210
         iscan = 1000
!
         !write(*,*) 'call adipls_get_one_el_info'
         call adipls_get_one_el_info( &
            s, l, nu1, nu2, iscan, R, G, M, &
            add_center_point, keep_surface_point, add_atmosphere, &
            do_restribute_mesh, store_for_adipls, &
            save_mode_info, order_to_save, save_mode_filename, &
            num, l_freq, l_inertia, l_order, l_em, ierr)
         !write(*,*) 'done adipls_get_one_el_info'
         if (ierr /= 0) then
            write(*,*) 'failed in adipls_get_one_el_info'
            stop 1
         end if
         if (expected_freq > 0) okay = .false.

         if(num.eq.0) goto 2010

      deallocate(l_freq, l_inertia, l_order, l_em)
2010   continue
      end subroutine get_adipls_frequency_info


      integer function how_many_extra_history_columns(s, id, id_extra)
         type (star_info), pointer :: s
         integer, intent(in) :: id, id_extra
         how_many_extra_history_columns = 0
      end function how_many_extra_history_columns
      
      
      subroutine data_for_extra_history_columns(s, id, id_extra, n, names, vals, ierr)
         type (star_info), pointer :: s
         integer, intent(in) :: id, id_extra, n
         character (len=maxlen_history_column_name) :: names(n)
         real(dp) :: vals(n)
         integer, intent(out) :: ierr
         ierr = 0
      end subroutine data_for_extra_history_columns

      
      integer function how_many_extra_profile_columns(s, id, id_extra)
         type (star_info), pointer :: s
         integer, intent(in) :: id, id_extra
         how_many_extra_profile_columns = 0
      end function how_many_extra_profile_columns
      
      
      subroutine data_for_extra_profile_columns(s, id, id_extra, n, nz, names, vals, ierr)
         type (star_info), pointer :: s
         integer, intent(in) :: id, id_extra, n, nz
         character (len=maxlen_profile_column_name) :: names(n)
         real(dp) :: vals(nz,n)
         integer, intent(out) :: ierr
         integer :: k
         ierr = 0
      end subroutine data_for_extra_profile_columns
      

      ! returns either keep_going or terminate.
      integer function extras_finish_step(s, id, id_extra)
         type (star_info), pointer :: s
         integer, intent(in) :: id, id_extra
         integer :: ierr
         extras_finish_step = keep_going
         call store_extra_info(s)
      end function extras_finish_step
      
      
      ! routines for saving and restoring extra data so can do restarts
         
         ! put these defs at the top and delete from the following routines
         !integer, parameter :: extra_info_alloc = 1
         !integer, parameter :: extra_info_get = 2
         !integer, parameter :: extra_info_put = 3
      
      
      subroutine alloc_extra_info(s)
         integer, parameter :: extra_info_alloc = 1
         type (star_info), pointer :: s
         call move_extra_info(s,extra_info_alloc)
      end subroutine alloc_extra_info
      
      
      subroutine unpack_extra_info(s)
         integer, parameter :: extra_info_get = 2
         type (star_info), pointer :: s
         call move_extra_info(s,extra_info_get)
      end subroutine unpack_extra_info
      
      
      subroutine store_extra_info(s)
         integer, parameter :: extra_info_put = 3
         type (star_info), pointer :: s
         call move_extra_info(s,extra_info_put)
      end subroutine store_extra_info
      
      
      subroutine move_extra_info(s,op)
         integer, parameter :: extra_info_alloc = 1
         integer, parameter :: extra_info_get = 2
         integer, parameter :: extra_info_put = 3
         type (star_info), pointer :: s
         integer, intent(in) :: op
         
         integer :: i, j, num_ints, num_dbls, ierr
         
         i = 0
         ! call move_int or move_flg    
         num_ints = i
         
         i = 0
         ! call move_dbl       
         
         num_dbls = i
         
         if (op /= extra_info_alloc) return
         if (num_ints == 0 .and. num_dbls == 0) return
         
         ierr = 0
         call star_alloc_extras(s% id, num_ints, num_dbls, ierr)
         if (ierr /= 0) then
            write(*,*) 'failed in star_alloc_extras'
            write(*,*) 'alloc_extras num_ints', num_ints
            write(*,*) 'alloc_extras num_dbls', num_dbls
            stop 1
         end if
         
         contains
         
         subroutine move_dbl(dbl)
            real(dp) :: dbl
            i = i+1
            select case (op)
            case (extra_info_get)
               dbl = s% extra_work(i)
            case (extra_info_put)
               s% extra_work(i) = dbl
            end select
         end subroutine move_dbl
         
         subroutine move_int(int)
            integer :: int
            i = i+1
            select case (op)
            case (extra_info_get)
               int = s% extra_iwork(i)
            case (extra_info_put)
               s% extra_iwork(i) = int
            end select
         end subroutine move_int
         
         subroutine move_flg(flg)
            logical :: flg
            i = i+1
            select case (op)
            case (extra_info_get)
               flg = (s% extra_iwork(i) /= 0)
            case (extra_info_put)
               if (flg) then
                  s% extra_iwork(i) = 1
               else
                  s% extra_iwork(i) = 0
               end if
            end select
         end subroutine move_flg
      
      end subroutine move_extra_info

      end module run_star_extras
      
